'use strict';

var React = require('react');
var classNames = require('classnames');

/**
 * simple abstraction for dragging events names
 */

var isTouchDevice = 'ontouchstart' in window // works on most browsers
 || 'onmsgesturechange' in window; // works on ie10 on ms surface

var dragEventFor = function () {
  var eventsFor = {
    touch: {
      start: 'touchstart',
      move: 'touchmove',
      end: 'touchend'
    },
    mouse: {
      start: 'mousedown',
      move: 'mousemove',
      end: 'mouseup'
    },
    pointer: {
      start: 'pointerdown',
      move: 'pointermove',
      end: 'pointerup'
    }
  };

  if ('onpointerdown' in window) {
    return eventsFor.pointer;
  } else {
    return eventsFor[isTouchDevice ? 'touch' : 'mouse'];
  }
}();

function getControlPosition(e) {
  var position = e.touches && e.touches[0] || e;
  return {
    clientX: position.clientX,
    clientY: position.clientY
  };
}

function addEvent(el, event, handler) {
  if (!el) {
    return;
  }
  if (el.attachEvent) {
    el.attachEvent('on' + event, handler);
  } else if (el.addEventListener) {
    el.addEventListener(event, handler, true);
  } else {
    el['on' + event] = handler;
  }
}

function removeEvent(el, event, handler) {
  if (!el) {
    return;
  }
  if (el.detachEvent) {
    el.detachEvent('on' + event, handler);
  } else if (el.removeEventListener) {
    el.removeEventListener(event, handler, true);
  } else {
    el['on' + event] = null;
  }
}

var ScrollView = React.createClass({
  displayName: 'ScrollView',


  propTypes: {
    className: React.PropTypes.string,
    itemAtPosition: React.PropTypes.func.isRequired,
    itemCount: React.PropTypes.number.isRequired,
    itemHeight: React.PropTypes.number.isRequired,
    itemPreloadCount: React.PropTypes.number,
    nativeScrollbar: React.PropTypes.bool,
    onScroll: React.PropTypes.func
  },

  getDefaultProps: function getDefaultProps() {
    return {
      nativeScrollbar: false,
      itemHeight: 50,
      itemCount: 0,
      itemPreloadCount: 10,
      itemAtPosition: function itemAtPosition(pos) {
        return React.createElement(
          'div',
          { key: pos },
          pos
        );
      }
    };
  },

  getInitialState: function getInitialState() {
    return {
      clientHeight: null,
      clientWidth: null,
      scrollTop: 0,
      scrolling: false
    };
  },

  componentDidMount: function componentDidMount() {
    window.addEventListener('resize', this.resize);
    this.resize();
  },

  componentWillUnmount: function componentWillUnmount() {
    window.removeEventListener('resize', this.resize);
  },

  resize: function resize() {
    var scrollbar = this.refs.scrollbar && React.findDOMNode(this.refs.scrollbar);
    this.setState({
      clientHeight: React.findDOMNode(this).clientHeight,
      clientWidth: React.findDOMNode(this).clientWidth,
      scrollbarHeight: scrollbar ? scrollbar.offsetHeight : 0,
      scrollbarInnerHeight: scrollbar ? scrollbar.clientHeight : 0
    });
  },

  scrollToItem: function scrollToItem(itemNumber) {

    var scrollTop = itemNumber * this.props.itemHeight;
    if (scrollTop < this.state.scrollTop || scrollTop > this.state.scrollTop + this.state.clientHeight) {
      //Only scroll if the item is not in view
      React.findDOMNode(this.refs.content).scrollTop = scrollTop;
      this.setState({
        scrollTop: scrollTop
      });
    }
  },

  _handleScroll: function _handleScroll() {
    if (this.props.onScroll) {
      this.props.onScroll();
    }
    if (this._scrollTimeout) {
      clearTimeout(this._scrollTimeout);
    }
    this._scrollTimeout = setTimeout(function () {
      this.setState({
        scrolling: false
      });
    }.bind(this), 100);
    this.setState({
      scrolling: true
    });
    return this.setState({
      scrollTop: React.findDOMNode(this.refs.content).scrollTop
    });
  },

  _onScrollbarStartDrag: function _onScrollbarStartDrag(ev) {
    ev.preventDefault();
    this.setState({ dragging: true });
    this.dragStartY = getControlPosition(ev).clientY;
    addEvent(window, dragEventFor.move, this._onScrollbarDrag);
    addEvent(window, dragEventFor.end, this._onScrollbarStopDrag);
  },

  _onScrollbarStopDrag: function _onScrollbarStopDrag(ev) {
    this.setState({ dragging: false });
    removeEvent(window, dragEventFor.move, this._onScrollbarDrag);
    removeEvent(window, dragEventFor.end, this._onScrollbarStopDrag);
  },

  _onScrollbarDrag: function _onScrollbarDrag(ev) {
    if (this.state.dragging) {
      var height = this.props.itemCount * this.props.itemHeight;
      var r = this.state.clientHeight / height;
      //get stuff
      var delta = getControlPosition(ev).clientY - this.dragStartY;
      React.findDOMNode(this.refs.content).scrollTop += delta / r;
      this.dragStartY += delta;
      this._handleScroll();
    }
  },

  _renderScrollBar: function _renderScrollBar(height) {

    if (this.props.nativeScrollbar) {
      return null;
    }

    var scrollbarToScrollerRatio = this.state.scrollbarHeight / this.state.clientHeight;
    var r = this.state.scrollbarHeight / (height * scrollbarToScrollerRatio);

    var scrollBarClasses = classNames({
      'pl-scroll-view-scroll-bar': true,
      'pl-enabled': height > this.state.clientHeight,
      'pl-dragging': this.state.dragging
    });
    var scrollBarHandleStyle = {
      height: this.state.scrollbarHeight * r - (this.state.scrollbarHeight - this.state.scrollbarInnerHeight),
      top: this.state.scrollTop * scrollbarToScrollerRatio * r
    };

    return React.createElement(
      'div',
      { className: scrollBarClasses, ref: 'scrollbar' },
      React.createElement('div', { className: 'pl-scroll-view-scroll-bar-handle',
        onMouseDown: this._onScrollbarStartDrag,
        style: scrollBarHandleStyle })
    );
  },

  render: function render() {

    var elements = [];
    var height = this.props.itemCount * this.props.itemHeight;

    var scrollerStyle = {
      height: height,
      width: this.state.clientWidth,
      pointerEvents: this.state.scrolling ? 'none' : ''
    };

    var elementCount = Math.ceil(this.state.clientHeight / this.props.itemHeight) + this.props.itemPreloadCount;

    var startingElement = Math.floor(this.state.scrollTop / this.props.itemHeight);
    startingElement = startingElement % 2 ? startingElement - 1 : startingElement; // Ensures that odd / even rows remain odd / even.
    startingElement = Math.max(startingElement, 0);

    var endingElement = Math.min(startingElement + elementCount, this.props.itemCount);

    for (var i = startingElement; i < endingElement; i++) {
      elements.push(this.props.itemAtPosition(i));
    }

    return React.createElement(
      'div',
      { className: 'pl-scroll-view' },
      React.createElement(
        'div',
        { className: 'pl-scroll-view-container' },
        React.createElement(
          'div',
          { className: 'pl-scroll-view-content',
            onScroll: this._handleScroll,
            ref: 'content',
            style: { width: this.state.clientWidth + 30 } },
          React.createElement(
            'div',
            { className: 'pl-scroll-view-scroller',
              ref: 'scroller',
              style: scrollerStyle },
            React.createElement('div', { className: 'pl-scroll-view-top-spaces',
              ref: 'spacer',
              style: { height: startingElement * this.props.itemHeight } }),
            elements
          )
        )
      ),
      this._renderScrollBar(height)
    );
  }

});

module.exports = ScrollView;
